@@ -21,8 +21,8 @@
  * CVS Information:
  * 
  * $Source: /home/globdev/CVS/globus-packages/gram/jobmanager/source/main.c,v $
- * $Date: 2011/01/28 00:56:13 $
- * $Revision: 1.22.2.10 $
+ * $Date: 2011/03/08 19:48:33 $
+ * $Revision: 1.22.2.11 $
  * $Author: bester $
  */
 
@@ -130,6 +130,11 @@ main(
     {
         reply_and_exit(NULL, rc, NULL);
     }
+
+    globus_thread_key_create(
+            &globus_i_gram_request_key,
+            NULL);
+
     rc = globus_gram_job_manager_logging_init(&config);
     if (rc != GLOBUS_SUCCESS)
     {
